
from collections import OrderedDict
import re

class tokenizemeta(type):
    @classmethod
    def __prepare__(meta, name, bases):
        return OrderedDict()

    @staticmethod
    def __new__(meta, clsname, bases, attrs):
        # Make named regex groups for all strings in the class body
        patterns = [ '(?P<%s>%s)' % (key, val) for key, val in attrs.items()
                     if isinstance(val, str) ]

        # Make the master regex pattern
        attrs['_pattern'] = re.compile('|'.join(patterns))
        return super().__new__(meta, clsname, bases, attrs)

class Tokenizer(metaclass=tokenizemeta):
    def tokenize(self, text):
        index = 0
        while index < len(text):
            m = self._pattern.match(text, index)
            if m:
                tokname = m.lastgroup
                toktext = m.group()
                if not tokname.startswith('ignore'):
                    yield (tokname, toktext)
                index = m.end()
            else:
                raise SyntaxError('Bad character %r' % text[index])

if __name__ == '__main__':
    class Example(Tokenizer):
        NAME = r'[a-zA-Z_][a-zA-Z0-9_]*'
        NUM = r'\d+'
        ASSIGN = r'='
        PLUS = r'\+'
        TIMES = r'\*'
        ignore = r'\s+'

    text = 'a = 3 + 4 * 5'
    tokenizer = Example()
    for tok in tokenizer.tokenize(text):
        print(tok)

